@@ -1,4 +1,4 @@
-__version__         = '0.4.7'
+__version__         = '0.4.8'
 __author__          = "Dave Martorana (http://davemartorana.com), Richard Cooper (http://frozenskys.com), Bill Jones (oraclebill), Dmitry Golomidov (deeGraYve)"
 __date__            = '2010-April-14'
 __url__             = 'http://postmarkapp.com'
@@ -470,8 +470,13 @@ class PMMail(object):
                     jsontxt = err.read().decode()
                     jsonobj = json.loads(jsontxt)
                     desc = jsonobj['Message']
-                except:
-                    desc = 'Description not given'
+                    error_code = jsonobj['ErrorCode']
+                except KeyError:
+                    raise PMMailUnprocessableEntityException('Unprocessable Entity: Description not given')
+
+                if error_code == 406:
+                    raise PMMailInactiveRecipientException('You tried to send email to a recipient that has been marked as inactive.')
+
                 raise PMMailUnprocessableEntityException('Unprocessable Entity: %s' % desc)
             elif err.code == 500:
                 raise PMMailServerErrorException('Internal server error at Postmark. Admins have been alerted.', err)
@@ -558,6 +563,9 @@ class PMBatchMail(object):
             message._check_values()
 
     def send(self, test=None):
+        # Has one of the messages caused an inactive recipient error?
+        inactive_recipient = False
+
         # Check messages for completeness prior to attempting to send
         self._check_values()
 
@@ -594,7 +602,7 @@ class PMBatchMail(object):
             # Attempt send
             try:
                 result = urlopen(req)
-                jsontxt = result.read()
+                jsontxt = result.read().decode()
                 result.close()
                 if result.code == 200:
                     results = json.loads(jsontxt)
@@ -607,11 +615,20 @@ class PMBatchMail(object):
                     raise PMMailUnauthorizedException('Sending Unauthorized - incorrect API key.', err)
                 elif err.code == 422:
                     try:
-                        jsontxt = err.read()
+                        jsontxt = err.read().decode()
                         jsonobj = json.loads(jsontxt)
                         desc = jsonobj['Message']
-                    except:
-                        desc = 'Description not given'
+                        error_code = jsonobj['ErrorCode']
+                    except KeyError:
+                        raise PMMailUnprocessableEntityException('Unprocessable Entity: Description not given')
+
+                    if error_code == 406:
+                        # One of the message recipients was inactive. Postmark still sends the
+                        # rest of the messages that have active recipients. Continue sending
+                        # the rest of the chunks.
+                        inactive_recipient = True
+                        continue
+
                     raise PMMailUnprocessableEntityException('Unprocessable Entity: %s' % desc)
                 elif err.code == 500:
                     raise PMMailServerErrorException('Internal server error at Postmark. Admins have been alerted.', err)
@@ -622,6 +639,10 @@ class PMBatchMail(object):
                     raise PMMailURLException('URLError: %d: The server couldn\'t fufill the request. (See "inner_exception" for details)' % err.code, err)
                 else:
                     raise PMMailURLException('URLError: The server couldn\'t fufill the request. (See "inner_exception" for details)', err)
+
+        if inactive_recipient:
+            raise PMMailInactiveRecipientException('You tried to send email to a recipient that has been marked as inactive.')
+
         return True
 
 
@@ -913,3 +934,11 @@ class PMMailURLException(PMMailSendException):
     have the base URLError object.
     '''
     pass
+
+class PMMailInactiveRecipientException(PMMailSendException):
+    '''
+    406: You tried to send a message to a recipient that has been marked as
+    inactive. If this was a batch operation, the rest of the messages were
+    still sent.
+    '''
+    pass
